////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// @file   mstar_ion.c
/// @brief  mstar ion driver
/// @author MStar Semiconductor Inc.
///
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include files
//-------------------------------------------------------------------------------------------------
#include <linux/err.h>
#include <linux/module.h>
#include <linux/ion.h>
#include "../ion_priv.h"

//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
#define NUM_HEAPS 1
static struct ion_device *idev;
static struct ion_heap *heaps[NUM_HEAPS];
static char * heap_system = "heap_system";


static int __init mod_ion_cust_init(void)
{
    struct ion_platform_heap heap_data;
    idev = ion_device_create(NULL);

    if (IS_ERR_OR_NULL(idev))
    {
        return PTR_ERR(idev);
    }

    /* create the heaps */
    heap_data.type = ION_HEAP_TYPE_SYSTEM;
    heap_data.id = 0;
    heap_data.name = heap_system; 
    heaps[0] = ion_heap_create(&heap_data);
    if (IS_ERR_OR_NULL(heaps[0])) 
    {
        pr_warn("%s(type:%d id:%d) isn't supported\n",
            heap_data.name, heap_data.type, heap_data.id);        
    }

    ion_device_add_heap(idev, heaps[0]);
    return 0;
}

static void __exit mod_ion_cust_exit(void)
{
	int i;

	ion_device_destroy(idev);
	for (i = 0; i < NUM_HEAPS; i++)
		ion_heap_destroy(heaps[i]);
}

late_initcall(mod_ion_cust_init);
module_exit(mod_ion_cust_exit);


MODULE_LICENSE("GPL");
MODULE_AUTHOR("MSTAR");
MODULE_DESCRIPTION("mstar ion customer driver");
